;+
;PROCEDURE: boxpos.pro
;PURPOSE: COMPUTE THE PLOTTING POSITIONS FOR (NROWS,NCOLS) OF PLOTS.
;
;USAGE: BOXPOS,NROWS=NROWS,NCOLS=NCOLS,PLSIZE=PLSIZE,SPACE=SPACE,POS=P $
;         ,MARG=MARG,VERBOSE=VERBOSE,INTRPT=INTRPT
;
;DESCRIPTION:
;      NROWS = # OF ROWS (REQUIRED INPUT)
;      NCOLS = # OF COLUMNS (REQUIRED INPUT)
;      PLSIZE = 2 ELEMENT ARRAY CONTAINING THE SIZE IN INCHES OF THE PLOT 
;               WINDOW (OPTIONAL INPUT: DEFAULT ASSUMES AN 8.5/11. ASPECT 
;               RATIO).
;      SPACE= 2 ELEMENT ARRAY CONTAINING THE X AND Y SPACING BETWEEN PLOTS 
;             (OPTIONAL INPUT: DEFAULT SPACE= 3*12PTS WHICH IS 3 TIMES THE 
;             TYPICAL CHARACTER SIZE).
;      MARG= 2 ELEMENT ARRAY CONTAINING THE PLOTTING WINDOW MARGINS IN UNITS 
;            CHARACTER SIZE. (OPTIONAL INPUT: DEFAULT MARG= [0,0]).
;      P = (NROWS,NCOLS) STRUCTURE ARRAY; POSITIONS ARE IN THE POS FIELD AND
;          ARE GIVEN IN NORMALIZED COORDINATES. (OUTPUT)
;-      
;==============================================================================
PRO BOXPOS,NROWS=NROWS,NCOLS=NCOLS,PLSIZE=PLSIZE,SPACE=SPACE,POS=P $
          ,MARG=MARG,VERBOSE=VERBOSE,INTRPT=INTRPT

IF NOT KEYWORD_SET(NROWS) THEN STOP,'NEED TO ENTER NUMBER OF ROWS'
IF NOT KEYWORD_SET(NCOLS) THEN STOP,'NEED TO ENTER NUMBER OF COLUMNS'
IF NOT KEYWORD_SET(PLSIZE) THEN PLSIZE=[8.5,11.]
IF NOT KEYWORD_SET(SPACE) THEN SPACE = [3.,3.]
IF NOT KEYWORD_SET(MARG) THEN MARG = [0.,0.]
MARG = MARG*12./72. ;CONVERT UNITS TO INCHES
SPACE = SPACE*12./72.
N = [NCOLS,NROWS]
DIM = NROWS*NCOLS

PSTN = {POS:FLTARR(4)}
P = REPLICATE(PSTN,NROWS,NCOLS)

LMIN = MIN(PLSIZE,IDMIN)
LMAX = MAX(PLSIZE,IDMAX)

LBOX = MIN([(LMIN-2*MARG(IDMIN)-(N(IDMIN)-1)*SPACE(IDMIN))/FLOAT(N(IDMIN)), $
       (LMAX-2*MARG(IDMAX)-(N(IDMAX)-1)*SPACE(IDMAX))/FLOAT(N(IDMAX))])


;DETERMINE LEFTOVER SPACE TO BE DIVIDED BETWEEN GAPS:
LXEXT = (PLSIZE(0) - (2*MARG(0)+N(0)*LBOX + (N(0) - 1)*SPACE(0)));/(N(0)-1)
LYEXT = (PLSIZE(1) - (2*MARG(1)+N(1)*LBOX + (N(1) - 1)*SPACE(1)));/(N(1)-1)

;NOW COMPUTE LOCATION OF A GIVEN PLOT:
FOR I = 1,N(1) DO BEGIN
   FOR J = 1,N(0) DO BEGIN
      P(I-1,J-1).POS = [(LXEXT/2.+MARG(0)+(J-1)*(LBOX+SPACE(0)))/PLSIZE(0) $
      ,(PLSIZE(1)-MARG(1)-I*LBOX-(I-1)*SPACE(1))/PLSIZE(1) $
      ,(LXEXT/2.+MARG(0)+(J-1)*(LBOX+SPACE(0)) + LBOX)/PLSIZE(0) $
      ,(PLSIZE(1)-MARG(1)-(I-1)*(LBOX+SPACE(1)))/PLSIZE(1)]

;      P(I-1,J-1).POS = [(LXEXT/2.+MARG(0)+(J-1)*(LBOX+SPACE(0)))/PLSIZE(0) $
;      ,(PLSIZE(1)-LYEXT/2.-MARG(1)-I*LBOX-(I-1)*SPACE(1))/PLSIZE(1) $
;      ,(LXEXT/2.+MARG(0)+(J-1)*(LBOX+SPACE(0)) + LBOX)/PLSIZE(0) $
;      ,(PLSIZE(1)-LYEXT/2.-MARG(1)-(I-1)*(LBOX+SPACE(1)))/PLSIZE(1)]

      IF KEYWORD_SET(VERBOSE) THEN $
       PRINT,I,J,[(LXEXT/2.+MARG(0)+(J-1)*(LBOX+SPACE(0))) $
      ,(PLSIZE(1)-LYEXT/2.-MARG(1)-I*LBOX-(I-1)*SPACE(1)) $
      ,(LXEXT/2.+MARG(0)+(J-1)*(LBOX+SPACE(0)) + LBOX) $
      ,(PLSIZE(1)-LYEXT/2.-MARG(1)-(I-1)*(LBOX+SPACE(1)))]


;      P(I-1,J-1).POS = [(MARG(0)+(J-1)*(LBOX+SPACE(0)+LXEXT))/PLSIZE(0) $
;      ,(PLSIZE(1)-MARG(1)-I*LBOX-(I-1)*(SPACE(1)+LYEXT))/PLSIZE(1) $
;      ,(MARG(0)+(J-1)*(LBOX+SPACE(0)+LXEXT) + LBOX)/PLSIZE(0) $
;      ,(PLSIZE(1)-MARG(1)-(I-1)*(LBOX+SPACE(1)+LYEXT))/PLSIZE(1)]


;      IF KEYWORD_SET(VERBOSE) THEN $
;       PRINT,I,J,[(MARG(0)+(J-1)*(LBOX+SPACE(0)+LXEXT)) $
;      ,(PLSIZE(1)-MARG(1)-I*LBOX-(I-1)*(SPACE(1)+LYEXT)) $
;      ,(MARG(0)+(J-1)*(LBOX+SPACE(0)+LXEXT) + LBOX) $
;      ,(PLSIZE(1)-MARG(1)-(I-1)*(LBOX+SPACE(1)+LYEXT))]

;   PRINT,I,J,P(I-1,J-1).POS
   ENDFOR
ENDFOR

IF KEYWORD_SET(INTRPT) THEN STOP

RET:
RETURN
END
